#region License Information
/* HeuristicLab
 * Copyright (C) 2002-2016 Heuristic and Evolutionary Algorithms Laboratory (HEAL)
 *
 * This file is part of HeuristicLab.
 *
 * HeuristicLab is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * HeuristicLab is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with HeuristicLab. If not, see <http://www.gnu.org/licenses/>.
 */
#endregion

using HeuristicLab.Common;
using HeuristicLab.Core;
using HeuristicLab.Encodings.SymbolicExpressionTreeEncoding;
using HeuristicLab.Persistence.Default.CompositeSerializers.Storable;
using HeuristicLab.Random;
namespace HeuristicLab.Problems.DataAnalysis.Symbolic {
  [StorableClass]
  public class FactorVariableTreeNode : SymbolicExpressionTreeTerminalNode {
    public new FactorVariable Symbol {
      get { return (FactorVariable)base.Symbol; }
    }
    [Storable]
    private string variableName;
    public string VariableName {
      get { return variableName; }
      set { variableName = value; }
    }

    [Storable]
    private string variableValue;
    public string VariableValue {
      get { return variableValue; }
      set { variableValue = value; }
    }

    [StorableConstructor]
    protected FactorVariableTreeNode(bool deserializing) : base(deserializing) { }
    protected FactorVariableTreeNode(FactorVariableTreeNode original, Cloner cloner)
      : base(original, cloner) {
      variableName = original.variableName;
      variableValue = original.variableValue;
    }
    protected FactorVariableTreeNode() { }
    public FactorVariableTreeNode(FactorVariable variableSymbol) : base(variableSymbol) { }

    public override bool HasLocalParameters {
      get { return true; }
    }

    public override void ResetLocalParameters(IRandom random) {
      base.ResetLocalParameters(random);
      variableName = Symbol.VariableNames.SampleRandom(random);
      variableValue = Symbol.GetVariableValues(VariableName).SampleRandom(random);
    }

    public override void ShakeLocalParameters(IRandom random, double shakingFactor) {
      base.ShakeLocalParameters(random, shakingFactor);
      if (random.NextDouble() < 0.2) {
        variableName = Symbol.VariableNames.SampleRandom(random);
      }
      variableValue = Symbol.GetVariableValues(VariableName).SampleRandom(random);
    }

    public override IDeepCloneable Clone(Cloner cloner) {
      return new FactorVariableTreeNode(this, cloner);
    }

    public override string ToString() {
      return variableName + " = " + variableValue;
    }
  }
}
